@@ -13,12 +13,8 @@
  * would do the trick for this but either makes the file large or truncates
  * symbols.  The approximate method seems to be plenty fast enough, costing
  * only about as much as one extra fstat() or so per process.
- *
- * Copyright (C) 1996 Charles Blake
- * Copyright (C) 1998 Michael K. Johnson
- * This file may be redistributed under the terms of the
- * GNU Library General Public License, see ../COPYING.LIB for precise terms.
  */
+#define _GNU_SOURCE /* for snprintf() prototype */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -35,8 +31,6 @@
 static char *sysmap, *sysmap_last, sysmap_fmt[10];
 static int   sysmap_len, sysmap_mean = 32, sysmap_addrsz;
 
-static char buf[128];
-
 /* scan backward in a string no further than address beg looking for c */
 static char *strchrrev(char *a, char *beg, char c) {
     if (a)
@@ -146,7 +140,6 @@ int open_psdb(void) {
 	"/boot/System.map-%s",
 	"/boot/System.map",
 	"/lib/modules/%s/System.map",
-	"/System.map",
 	NULL
     };
     static char *psdb_paths[] = {
@@ -194,7 +187,7 @@ void close_psdb(void) {
 
 int read_tbl(struct dbtbl_s *dbtbl, struct tbl_s *tbl) {
     lseek(psdb, dbtbl->off, SEEK_SET);
-    tbl->tbl = (struct sym_s *) xmalloc(dbtbl->size);
+    tbl->tbl = (struct sym_s *) malloc(dbtbl->size);
     if (read(psdb, (char *) tbl->tbl, dbtbl->size) != dbtbl->size) {
 	perror(PSDATABASE);
 	exit(1);
@@ -204,31 +197,6 @@ int read_tbl(struct dbtbl_s *dbtbl, struct tbl_s *tbl) {
     return 0;
 }
 
-/* Uhhh: This is a linear search. */
-char* find_module_sym(unsigned long address) {
-   FILE *fd;
-   unsigned long adr, best_adr = 0;
-   char mod[32], nm[32]; char c;
-   *mod = 1;
-   *buf = 0; /* sprintf (buf, "%08x", address); */
-   fd = fopen ("/proc/ksyms", "r");
-   if (!fd) return ("(no proc)");
-   while (!feof (fd) && !(*mod == 0)) {
-      fscanf (fd, "%lx %s\n", &adr, nm);
-      if ((c = fgetc (fd)) != '[') *mod = 0; 
-      else { fscanf (fd, "%s", mod); };
-      if (abs(address - adr) < abs(address - best_adr) /* && address >= adr */
-	  && abs(address - adr) < 0xa00) {
-	 best_adr = adr; *buf = '[';
-	 if (address - adr > 0x100) strcpy (buf+1, mod); /* module name */
-	 else strcpy(buf, nm); /* name of function/variable */
-      };
-   };
-   fclose (fd);
-   return buf;
-};
-      
-
 char * find_func(unsigned long address)	{
     int n;
     struct sym_s *p;
@@ -242,7 +210,6 @@ char * find_func(unsigned long address)	{
 	read_tbl(&db_hdr.fncs, &fncs);
     p = fncs.tbl;
     n = fncs.nsym;
-    if (address > p[n-1].addr) return find_module_sym(address);
     while (n) {
 	int i = n / 2;
 	if (p[i].addr < address) {
@@ -276,7 +243,7 @@ char * wchan(unsigned long address) {
 
 #ifdef SYSMAP_TEST
 int main(int ac, char** av) {
-    if (ac < 3) {printf("%s System.map lines hexaddr ...\n",av[0]); return 1;}
+    if (ac < 3) {printf("%s System.map linesz hexaddr ...\n",av[0]); return 1;}
     if (!sysmap_mmap(av[1])) return 1;
     if ((sysmap_mean = atoi(av[2])) <= 0) return 1;
     for (av += 3; *av; av++)
