@@ -1,11 +1,19 @@
 /* The shadow of the original with only common prototypes now. */
 
+#include <proc/libc6-protect.h>
+
 /* do not include the kernel inline definitions of string functions */
 #define _LINUX_STRING_H_ 
 
 #include <stdio.h>
 #include <sys/types.h>
+#if !defined(__GLIBC__)
 #include <linux/sched.h>
+#else /* __GLIBC__ */
+#include <asm/page.h>
+#include <sys/ioctl.h>
+#include <asm/param.h>
+#endif /* __GLIBC__ */
 
 char *wchan(unsigned long);
 char *find_func(unsigned long address);
