@@ -1,9 +1,6 @@
 /*
  * New Interface to Process Table -- PROCTAB Stream (a la Directory streams)
- * Copyright (C) 1996 Charles L. Blake.
- * Copyright (C) 1998 Michael K. Johnson
- * May be distributed under the conditions of the
- * GNU Library General Public License; a copy is in ../COPYING.LIB
+ * Copyright(C) 1996. Charles L. Blake.
  */
 #include "proc/version.h"
 #include "proc/readproc.h"
@@ -25,24 +22,26 @@
  */
 PROCTAB* openproc(int flags, ...) {
     va_list ap;
-    PROCTAB* PT = xmalloc(sizeof(PROCTAB));
+    PROCTAB* PT = malloc(sizeof(PROCTAB));
 
     if (!Do(PID) && !(PT->procfs = opendir("/proc")))
     	return NULL;
+    if (Do(REAL)) flags |= PROC_FILLSTATUS;
     PT->flags = flags;
     va_start(ap, flags);		/*  Init args list */
-    if (Do(PID))
-    	PT->pids = va_arg(ap, pid_t*);
-    else if (Do(TTY))
+    if (Do(PID)) {
+        PT->pids = va_arg(ap, pid_t*);
+    } else if (Do(NOTPID)) {
+        PT->pids = va_arg(ap, pid_t*);
+        PT->nuid = va_arg(ap, int);
+    } else if (Do(TTY))
     	PT->ttys = va_arg(ap, dev_t*);
-    else if (Do(UID)) {
+    else if (Do(UID) || Do(NOTUID) || Do(GID) || Do(NOTGID)) {
     	PT->uids = va_arg(ap, uid_t*);
 	PT->nuid = va_arg(ap, int);
     } else if (Do(STAT))
     	PT->stats = va_arg(ap, char*);
     va_end(ap);				/*  Clean up args list */
-    if (Do(ANYTTY) && Do(TTY))
-	PT->flags = PT->flags & ~PROC_TTY; /* turn off TTY flag */
     return PT;
 }
 
@@ -67,6 +66,29 @@ void freeproc(proc_t* p) {
     free(p);
 }
 
+void status2proc (char* S, proc_t* P, int fill) {
+    char* tmp;
+
+    if (fill == 1) {
+        memset(P->cmd, 0, sizeof P->cmd);
+        sscanf (S, "Name:\t%39c", P->cmd);
+        tmp = strchr(P->cmd,'\n');
+        *tmp='\0';
+	tmp = strstr (S,"State");
+        sscanf (tmp, "State:\t%c", &P->state);
+    }
+    tmp = strstr (S,"Pid:");
+    sscanf (tmp, 
+        "Pid:\t%d\nPPid:\t%d\nUid:\t%d\t%d\t%d\t%d\nGid:\t%d\t%d\t%d\t%d\n"
+        "VmSize: %lu kB\nVmLck: %lu kB\nVmRSS: %lu kB\nVmData: %lu kB\n"
+        "VmStk: %lu kB\nVmExe: %lu kB\nVmLib: %lu kB\n"
+        "SigPnd: %lx\nSigBlk: %lx\nSigIgn: %lx\nSigCgt: %lx",
+        &P->pid, &P->ppid, &P->ruid, &P->uid, &P->suid, &P->fsuid,
+        &P->rgid, &P->gid, &P->sgid, &P->fsgid, &P->vm_size, &P->vm_lock,
+	&P->vm_rss, &P->vm_data, &P->vm_stack, &P->vm_exe, &P->vm_lib,
+	&P->signal, &P->blocked, &P->sigignore, &P->sigcatch);
+}
+
 /* stat2proc() makes sure it can handle arbitrary executable file basenames
    for `cmd', i.e. those with embedded whitespace or embedded ')'s.  Such names
    confuse %s (see scanf(3)), so the string is split and %39c is used instead.
@@ -79,8 +101,8 @@ void stat2proc(char* S, proc_t* P) {
     memset(P->cmd, 0, sizeof P->cmd);	/* clear even though *P xcalloc'd ?! */
     sscanf(S, "%d (%39c", &P->pid, P->cmd);
     sscanf(tmp + 2,			/* skip space after ')' too */
-	   "%c %d %d %d %d %d %lu %lu %lu %lu %lu %ld %ld %ld %ld %d "
-	   "%d %lu %lu %ld %lu %lu %lu %lu %lu %lu %lu %lu %Lu %Lu %Lu %Lu %lu",
+	   "%c %d %d %d %d %d %lu %lu %lu %lu %lu %ld %ld %ld %ld %ld "
+	   "%ld %lu %lu %ld %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu",
 	   &P->state, &P->ppid, &P->pgrp, &P->session, &P->tty, &P->tpgid,
 	   &P->flags, &P->min_flt, &P->cmin_flt, &P->maj_flt, &P->cmaj_flt,
 	   &P->utime, &P->stime, &P->cutime, &P->cstime, &P->priority, &P->nice,
@@ -99,7 +121,7 @@ void stat2proc(char* S, proc_t* P) {
 }
 
 void statm2proc(char* s, proc_t* P) {
-    sscanf(s, "%ld %ld %ld %ld %ld %ld %ld",
+    sscanf(s, "%d %d %d %d %d %d %d",
 	   &P->size, &P->resident, &P->share,
 	   &P->trs, &P->lrs, &P->drs, &P->dt);
 }
@@ -117,10 +139,7 @@ int file2str(char *directory, char *what, char *ret, int cap) {
 
     sprintf(filename, "%s/%s", directory, what);
     if ( (fd       = open(filename, O_RDONLY, 0)) == -1 ) return -1;
-    if ( (num_read = read(fd, ret, cap - 1))      <= 0 ) {
-	close (fd);
-	return -1;
-    }
+    if ( (num_read = read(fd, ret, cap - 1))      <= 0 ) return -1;
     ret[num_read] = 0;
     close(fd);
     return num_read;
@@ -148,7 +167,7 @@ char** file2strvec(char* directory, char* what) {
 	}
 	if (end_of_file && buf[n-1])		/* last read char not null */
 	    buf[n++] = '\0';			/* so append null-terminator */
-	rbuf = xrealloc(rbuf, tot + n);		/* allocate more memory */
+	rbuf = realloc(rbuf, tot + n);		/* allocate more memory */
 	memcpy(rbuf + tot, buf, n);		/* copy buffer into it */
 	tot += n;				/* increment total byte ctr */
 	if (end_of_file)
@@ -166,7 +185,7 @@ char** file2strvec(char* directory, char* what) {
 	    c += sizeof(char*);
     c += sizeof(char*);				/* one extra for NULL term */
 
-    rbuf = xrealloc(rbuf, tot + c + align);	/* make room for ptrs AT END */
+    rbuf = realloc(rbuf, tot + c + align);	/* make room for ptrs AT END */
     endbuf = rbuf + tot;			/* addr just past data buf */
     q = ret = (char**) (endbuf+align);		/* ==> free(*ret) to dealloc */
     *q++ = p = rbuf;				/* point ptrs to the strings */
@@ -216,7 +235,7 @@ char** file2strvec(char* directory, char* what) {
 proc_t* readproc(PROCTAB* PT, proc_t* rbuf) {
     static struct direct *ent;		/* dirent handle */
     static struct stat sb;		/* stat buffer */
-    static char path[32], sbuf[256];	/* bufs for stat,statm */
+    static char path[32], sbuf[512];	/* bufs for stat,statm */
     int allocated = 0, matched = 0;	/* flags */
     proc_t *p = NULL;
 
@@ -234,51 +253,98 @@ next_proc:				/* get next PID for consideration */
 	    ;
 	if (!ent || !ent->d_name)
 	    return NULL;
+        if (Do(NOTPID) && XinLN (pid_t, (pid_t) atoi(ent->d_name), 
+                                 PT->pids, PT->nuid))
+            goto next_proc;
 	sprintf(path, "/proc/%s", ent->d_name);
     }
     if (stat(path, &sb) == -1)		/* no such dirent (anymore) */
 	goto next_proc;
-    if (Do(UID) && !XinLN(uid_t, sb.st_uid, PT->uids, PT->nuid))
-	goto next_proc;			/* not one of the requested uids */
+    if (!Do(REAL)){
+        if (Do(UID) && !XinLN(uid_t, sb.st_uid, PT->uids, PT->nuid))
+            goto next_proc;		/* not one of the requested uids */
+        if (Do(NOTUID) && XinLN(uid_t, sb.st_uid, PT->uids, PT->nuid))
+            goto next_proc;		/* not one of the requested uids */
+        if (Do(GID) && !XinLN(uid_t, sb.st_gid, PT->uids, PT->nuid))
+            goto next_proc;		/* not one of the requested uids */
+        if (Do(NOTGID) && XinLN(uid_t, sb.st_gid, PT->uids, PT->nuid))
+            goto next_proc;		/* not one of the requested uids */
+    }
 
-    if (!allocated) {				 /* assign mem for return buf */
-	p = rbuf ? rbuf : xcalloc(p, sizeof *p); /* passed buf or alloced mem */
-	allocated = 1;				 /* remember space is set up */
+    if (!allocated) {			 /* assign mem for return buf */
+	p = rbuf ? rbuf : xcalloc(p, sizeof *p);/* passed buf or alloced mem */
+	allocated = 1;			 /* remember space is set up */
     }
     p->uid = sb.st_uid;			/* need a way to get real uid */
 
-    if ((file2str(path, "stat", sbuf, sizeof sbuf)) == -1)
-	goto next_proc;			/* error reading /proc/#/stat */
-    stat2proc(sbuf, p);				/* parse /proc/#/stat */
-
-    if (!matched && Do(TTY) && !XinL(dev_t, p->tty, PT->ttys))
-	goto next_proc;			/* not one of the requested ttys */
+    if (Do(FILLSTAT)) {
+        if ((file2str(path, "stat", sbuf, sizeof sbuf)) == -1)
+            goto next_proc;               /* error reading /proc/#/stat */
+        stat2proc(sbuf, p);                       /* parse /proc/#/stat */
+        if (Do(FILLSTATUS)) {
+            if ((file2str(path, "status", sbuf, sizeof sbuf)) == -1)
+                goto next_proc;		/* error reading /proc/#/status */
+            status2proc(sbuf, p, 0);            /* parse /proc/#/status */
+        }
+    } else if (Do(FILLSTATUS)) {
+            if ((file2str(path, "status", sbuf, sizeof sbuf)) == -1)
+                goto next_proc;  
+            status2proc(sbuf, p, 1);
+    }
 
     if (!matched && Do(ANYTTY) && p->tty == -1)
 	goto next_proc;			/* no controlling terminal */
 
+    if (!matched && Do(TTY) && !XinL(dev_t, p->tty, PT->ttys))
+	goto next_proc;			/* not one of the requested ttys */
+
     if (!matched && Do(STAT) && !strchr(PT->stats,p->state))
 	goto next_proc;			/* not one of the requested states */
 
+    if (!matched && Do(NOSLEAD) && p->session == p->pid)
+        goto next_proc;
+    
+    if (!matched && Do(NOGLEAD) && p->pgrp == p->pid)
+        goto next_proc;
+
+    if (!matched && Do(REAL)){
+        if (Do(UID)   && !XinLN(uid_t, p->ruid, PT->uids, PT->nuid))
+            goto next_proc;		/* not one of the requested uids */
+        if (Do(NOTUID) && XinLN(uid_t, p->ruid, PT->uids, PT->nuid))
+            goto next_proc;		/* not one of the requested uids */
+        if (Do(GID)   && !XinLN(uid_t, p->rgid, PT->uids, PT->nuid))
+            goto next_proc;		/* not one of the requested uids */
+        if (Do(NOTGID) && XinLN(uid_t, p->rgid, PT->uids, PT->nuid))
+            goto next_proc;		/* not one of the requested uids */
+    }
+    
     if (Do(FILLMEM)) {				/* read, parse /proc/#/statm */
 	if ((file2str(path, "statm", sbuf, sizeof sbuf)) != -1 )
 	    statm2proc(sbuf, p);		/* ignore statm errors here */
-	else					/* statm fields just zero */
-	    p->size = p->resident = p->share = p->trs =
-		p->lrs = p->drs = p->dt = 0.0;	/* rbuf not necessarily 0 */
-    }
+    }						/* statm fields just zero */
 
     /* some number->text resolving which is time consuming */
     if (Do(FILLTTY))
 	dev_to_tty(p->ttyc, p->tty);
-    if (Do(FILLUSR))
-	strncpy(p->user, user_from_uid(p->uid), sizeof p->user);
-
-    if (Do(FILLCMD))				/* read+parse /proc/#/cmdline */
-	p->cmdline = file2strvec(path, "cmdline");
-    if (Do(FILLENV))				/* read+parse /proc/#/environ */
-	p->environ = file2strvec(path, "environ");
+    if (Do(FILLUSR)) {
+        strncpy(p->user, user_from_uid(p->uid), sizeof p->user);
+        strncpy(p->group, group_from_gid(p->gid), sizeof p->group);
+        if(Do(FILLSTATUS)) {
+            strncpy(p->ruser, user_from_uid(p->ruid), sizeof p->ruser);
+            strncpy(p->suser, user_from_uid(p->suid), sizeof p->suser);
+            strncpy(p->fsuser, user_from_uid(p->fsuid), sizeof p->fsuser);
+            strncpy(p->rgroup, group_from_gid(p->rgid), sizeof p->rgroup);
+            strncpy(p->sgroup, group_from_gid(p->sgid), sizeof p->sgroup);
+            strncpy(p->fsgroup, group_from_gid(p->fsgid), sizeof p->fsgroup);
+        }
+    }
     
+    if (Do(FILLCMD)) {			/* read+parse /proc/#/cmdline */
+	p->cmdline = file2strvec(path, "cmdline");
+    }
+    if (Do(FILLENV)) {			/* read+parse /proc/#/environ */
+	p->environ = file2strvec(path, "environ");
+    }
     if (p->state == 'Z')			/* fixup cmd for zombies */
 	strncat(p->cmd," <zombie>", sizeof p->cmd);
 
@@ -319,17 +385,8 @@ proc_t* readproctree(int flags, ...) {
     
     /* pass through apropriate arguments to openproc */
     va_start(ap, flags);
-    if (Do(UID)) {
-	/* temporary variables to ensure that va_arg() instances
-	 * are called in the right order
-	 */
-	uid_t* u;
-	int i;
-
-	u = va_arg(ap, uid_t*);
-	i = va_arg(ap, int);
-	PT = openproc(flags, u, i);
-    }
+    if (Do(UID))
+	PT = openproc(flags, va_arg(ap, uid_t*), va_arg(ap, int));
     else if (Do(PID) || Do(TTY) || Do(STAT))
 	PT = openproc(flags, va_arg(ap, void*));
     else
@@ -354,7 +411,6 @@ proc_t* readproctree(int flags, ...) {
 	    tmp2->r = tmp->l->r;		/* insert as child of found node */
 	    tmp->l->r = node;
 	}
-    closeproc(PT);
     return &tree;
 }
 
@@ -370,27 +426,17 @@ proc_t** readproctab(int flags, ...) {
     va_list ap;
 
     va_start(ap, flags);		/* pass through args to openproc */
-    if (Do(UID)) {
-	/* temporary variables to ensure that va_arg() instances
-	 * are called in the right order
-	 */
-	uid_t* u;
-	int i;
-
-	u = va_arg(ap, uid_t*);
-	i = va_arg(ap, int);
-	PT = openproc(flags, u, i);
-    }
+    if (Do(UID))
+	PT = openproc(flags, va_arg(ap, uid_t*), va_arg(ap, int));
     else if (Do(PID) || Do(TTY) || Do(STAT))
 	PT = openproc(flags, va_arg(ap, void*)); /* assume ptr sizes same */
     else
 	PT = openproc(flags);
     va_end(ap);
     do {					/* read table: */
-	tab = xrealloc(tab, (n+1)*sizeof(proc_t*));/* realloc as we go, using */
+	tab = realloc(tab, (n+1)*sizeof(proc_t*));/* realloc as we go, using */
 	tab[n] = readproc(PT, NULL);		  /* final null to terminate */
     } while (tab[n++]);				  /* stop when NULL reached */
-    closeproc(PT);
     return tab;
 }
 
