@@ -968,7 +968,8 @@ int show_task_info (proc_t * task, int main_mem, int current_time, dopts *opts)
 	case P_WCHAN:
  	    if (TEST(WCHAN))
 		n = print_str (wchan (task->wchan), max);
-	    else 
+	    else /* just the lower 32 bits of a kernel address are really
+                    interesting */
 		n = print_hex (task->wchan & 0xffffffff, max);
 	    break;
         case P_SIGNAL:
