@@ -18,16 +18,18 @@
  *
  * Changes Copyright (C) 1993, 1994 Michael K. Johnson,
  *   and   Copyright (C) 1995, 1996 Charles Blake,
- *   and   Copyright (C) 1996 Helmut Geyer
+ *   and   Copyright (C) 1996, 1997 Helmut Geyer
  * See file COPYING for copyright details.
  */
-#include <proc/version.h>
-#include <proc/readproc.h>
-#include <proc/ps.h>
-#include <proc/psdata.h>
-#include <proc/devname.h>
-#include <proc/tree.h>
-#include <proc/sysinfo.h>
+#include "proc/version.h"
+#include "proc/readproc.h"
+#include "proc/ps.h"
+#include "proc/psdata.h"
+#include "proc/devname.h"
+#include "proc/tree.h"
+#include "proc/sysinfo.h"
+#include "proc/showtask.h"
+#include "proc/output.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -42,13 +44,9 @@
 #include <sys/types.h>
 #include <sys/ioctl.h>
 #include <sys/resource.h>
-#include <linux/sched.h>
-#include <linux/tty.h>
 #include <getopt.h>
 #include <errno.h>
 
-#include "proc/showtask.h"
-#include "proc/output.h"
 
 #define BSD_PERSON 1
 #define POSIX_PERSON 2
