@@ -1,6 +1,6 @@
 /* fuser.c - identify processes using files */
 
-/* Copyright 1993-1996 Werner Almesberger. See file COPYING for details. */
+/* Copyright 1993-1997 Werner Almesberger. See file COPYING for details. */
 
 
 #include <stdlib.h>
@@ -15,9 +15,9 @@
 #include <netdb.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <asm/byteorder.h> /* really needed ??*/
 #include <netinet/in.h>
-#include <linux/fs.h> /* for MKDEV */
+#include <arpa/inet.h>
+#include <linux/kdev_t.h> /* for MKDEV */
 
 #include "comm.h"
 #include "signals.h"
@@ -43,7 +43,9 @@
 
 
 typedef struct _net_cache {
-    int port;
+    int lcl_port;
+    int rmt_port;
+    unsigned long rmt_addr;
     ino_t ino;
     struct _net_cache *next;
 } NET_CACHE;
@@ -96,7 +98,6 @@ static void fill_net_cache(SPACE_DSC *dsc)
     FILE *file;
     NET_CACHE *new,*last;
     char buffer[PATH_MAX+1],line[MAX_LINE+1];
-    int port,ino;
 
     if (dsc->once) return;
     dsc->once = 1;
@@ -108,12 +109,20 @@ static void fill_net_cache(SPACE_DSC *dsc)
     last = NULL;
     (void) fgets(line,MAX_LINE,file);
     while (fgets(line,MAX_LINE,file)) {
-	if (sscanf(line,"%*d: %*x:%x %*x:%*x %*x %*x:%*x %*x:%*x %*x %*d %*d "
-	  "%d",&port,&ino) != 2) continue;
-	if (!ino) continue;
 	new = malloc(sizeof(NET_CACHE));
-	new->port = port;
-	new->ino = ino;
+	if (!new) {
+	    perror("malloc");
+	    exit(1);
+	}
+	if (sscanf(line,"%*d: %*x:%x %lx:%x %*x %*x:%*x %*x:%*x %*x %*d %*d "
+	  "%ld",&new->lcl_port,&new->rmt_addr,&new->rmt_port,&new->ino) != 4) {
+	    free(new);
+	    continue;
+	}
+	if (!new->ino) {
+	    free(new);
+	    continue;
+	}
 	new->next = NULL;
 	if (last) last->next = new;
 	else dsc->cache = new;
@@ -283,7 +292,7 @@ static void show_files(void)
     const struct passwd *pw;
     const char *name,*scan;
     char tmp[10],path[PATH_MAX+1],comm[COMM_LEN+1];
-    int length,header,first,dummy;
+    int length,header,first,dummy,last_namelen;
     pid_t self;
 
     self = getpid();
@@ -295,7 +304,7 @@ static void show_files(void)
 		header = 0;
 	    }
 	    length = 0;
-	    for (scan = file->name; *scan; scan++)
+	    for (scan = file->name; scan && *scan; scan++)
 		if (*scan == '\\') length += printf("\\\\");
 		else if (*scan > ' ' && *scan <= '~') {
 			putchar(*scan);
@@ -304,6 +313,10 @@ static void show_files(void)
 		    else length += printf("\\%03o",*scan);
 	    if (file->name_space)
 		length += printf("/%s",file->name_space->name);
+            if (length != 0) 
+                last_namelen=length;
+            else
+                printf("%*.*s",last_namelen,last_namelen,"                                                                              ");
 	    if (!(file->flags & FLAG_VERB)) {
 		putchar(':');
 		length++;
@@ -397,6 +410,57 @@ static void enter_item(const char *name,int flags,int sig_number,dev_t dev,
 }
 
 
+static int parse_inet(const char *spec,const char *name_space,int *lcl_port,
+  unsigned long *rmt_addr,int *rmt_port)
+{
+    char *s,*here,*next,*end;
+    int port,field;
+
+    if (!(s = strdup(spec))) {
+	perror("strdup");
+	exit(1);
+    }
+    *lcl_port = *rmt_port = -1;
+    *rmt_addr = 0;
+    field = 0;
+    for (here = s; here; here = next ? next+1 : NULL) {
+	next = strchr(here,',');
+	if (next) *next = 0;
+	switch (field) {
+	    case 0:
+		/* fall through */
+	    case 2:
+		if (!*here) break;
+		port = strtoul(here,&end,0);
+		if (*end) {
+		    struct servent *se;
+
+		    if (!(se = getservbyname(here,name_space)))
+			return 0;
+		    port = ntohs(se->s_port);
+		}
+		if (field) *rmt_port = port;
+		else *lcl_port = port;
+		break;
+	    case 1:
+		if (!*here) break;
+		if ((long) (*rmt_addr = inet_addr(here)) == -1) {
+		    struct hostent *hostent;
+
+		    if (!(hostent = gethostbyname(here))) return 0;
+		    if (hostent->h_addrtype != AF_INET) return 0;
+		    memcpy(rmt_addr,hostent->h_addr,hostent->h_length);
+		}
+		break;
+	    default:
+		return 0;
+	}
+	field++;
+    }
+    return 1;
+}
+
+
 static void usage(void)
 {
     fprintf(stderr,"usage: fuser [ -a | -q ] [ -n space ] [ -signal ] "
@@ -535,23 +599,20 @@ int main(int argc,char **argv)
 	    }
 	    else {
 		NET_CACHE *walk;
-		int port;
-		char *end;
+		unsigned long rmt_addr;
+		int lcl_port,rmt_port;
 
 		fill_net_cache(this_name_space);
-		port = strtoul(*argv,&end,0);
-		if (*end) {
-		    struct servent *se;
-
-		    if (!(se = getservbyname(*argv,this_name_space->name))) {
-			fprintf(stderr,"%s/%s: invalid port\n",*argv,
-			  this_name_space->name);
-			continue;
-		    }
-		    port = ntohs(se->s_port);
+		if (!parse_inet(*argv,this_name_space->name,&lcl_port,
+		  &rmt_addr,&rmt_port)) {
+		    fprintf(stderr,"%s/%s: invalid specificiation\n",*argv,
+		      this_name_space->name);
+		    continue;
 		}
 		for (walk = this_name_space->cache; walk; walk = walk->next)
-		    if (walk->port == port)
+		    if ((lcl_port == -1 || walk->lcl_port == lcl_port) &&
+		      (!rmt_addr || walk->rmt_addr == rmt_addr) &&
+		      (rmt_port == -1 || walk->rmt_port == rmt_port))
 			enter_item(*argv,flags,sig_number,0,walk->ino,
 			    this_name_space);
 	    }
