@@ -4,6 +4,8 @@
  * Copyright (c) 1992 Branko Lankester
  * Copyright (c) 1992 Roger Binns
  *
+ * Copyrigth (c) 1995-1997 Helmut Geyer
+ *
  * Snarfed and HEAVILY modified for the YAPPS (yet another /proc ps)
  * by Michael K. Johnson, johnsonm@sunsite.unc.edu.  What is used is what
  * is required to have a common interface.
@@ -70,8 +72,6 @@
 #include <time.h>
 #include <sys/ioctl.h>
 #include <pwd.h>
-#include <linux/sched.h>
-#include <linux/tty.h>
 #include <termcap.h>
 #include <termios.h>
 #include <signal.h>
@@ -89,6 +89,7 @@
 #include "proc/readproc.h"
 #include "proc/showtask.h"
 #include "proc/devname.h"
+
 /*
  * these should be in the readproc.h header or in the ps.h header 
  */
@@ -267,7 +268,7 @@ void get_options (void)
     } else {
 	if (fgets (Fields, 2046, fp) != NULL) {
 	    pt = strstr (Fields, "\n");
-	    *pt = 0;
+	    if (pt != NULL) *pt = 0; /* This may happen with a broken .toprc */
 	}
 	fgets (Options, 254, fp);
 	fclose (fp);
@@ -355,7 +356,10 @@ int main (int argc, char **argv)
     char c;    /* The key read in. */
     char *p;           /* for environment variables */
     unsigned int i;
-    
+    if (atexit(clean_up)) {
+        fprintf (stderr, "couldnt register clean_up\n");
+        exit(1);
+    }
     get_options ();
     p=getenv("TOP_FORMAT");
     if (p!=NULL) strcpy(Fields,p);
@@ -553,17 +557,21 @@ int main (int argc, char **argv)
  * ####################################################################### 
  */
 
-
-	/*
-	 *  end when exiting with an error.
-	 */
-void error_end (int rno)
+void clean_up()
 {
     if (DTEST(WCHAN))
 	close_psdb ();
     ioctl (0, TCSETSF, &Savetty);
     PUTP (tgoto (cm, 0, topsetup.Lines - 1));
     fputs ("\r\n", stdout);
+    PUTP (me);
+}
+
+	/*
+	 *  end when exiting with an error.
+	 */
+void error_end (int rno)
+{
     exit (rno);
 }
 /*
@@ -571,11 +579,6 @@ void error_end (int rno)
  */
 void end (void)
 {
-    if (DTEST(WCHAN))
-	close_psdb ();
-    ioctl (0, TCSETSF, &Savetty);
-    PUTP (tgoto (cm, 0, topsetup.Lines - 1));
-    fputs ("\r\n", stdout);
     exit (0);
 }
 
@@ -1227,8 +1230,8 @@ void do_stats (proc_t ** p, float elapsed_time, int pass)
     int system_ticks = 0, user_ticks = 0, nice_ticks = 0, idle_ticks = 1000;
     static int prev_count = 0;
     int stime, utime;
-    static struct save_hist save_history[NR_TASKS];
-    struct save_hist New_save_hist[NR_TASKS];
+    static struct save_hist save_history[MAX_NR_TASKS];
+    static struct save_hist New_save_hist[MAX_NR_TASKS];
 
     /*
      * Make a pass through the data to get stats.
@@ -1278,7 +1281,7 @@ void do_stats (proc_t ** p, float elapsed_time, int pass)
 		stime -= save_history[i].stime;
 		utime -= save_history[i].utime;
 
-		i = NR_TASKS;
+		i = MAX_NR_TASKS;
 	    }
 	    i++;
 	}
@@ -1301,7 +1304,7 @@ void do_stats (proc_t ** p, float elapsed_time, int pass)
 
 	index++;
 	n++;
-	if (n > NR_TASKS) {
+	if (n > MAX_NR_TASKS) {
 	    printf (PROGNAME ": Help!  Too many tasks!\n");
 	    end ();
 	}
