@@ -7,6 +7,7 @@
 proc_t** readproctab2(int flags, proc_t** tab, void *args, int N);
 void parse_options(char *Options, int secure);
 void get_options(void);
+void clean_up(void);
 void error_end(int rno);
 void end(void);
 void stop(void);
@@ -31,6 +32,7 @@ unsigned show_meminfo(void);
 void do_stats(proc_t** p, float elapsed_time, int pass);
 void do_key(char c);
 
+#define MAX_NR_TASKS 4096
 
 	/* Name of the config file (in $HOME)  */
 #ifndef RCFILE
