@@ -87,7 +87,7 @@ int main(int argc, char **argv) {
 	fprintf(stderr, "%d column window is too narrow\n", maxcmd);
 	exit(1);
     }
-    maxcmd -= 29 + (from ? 16 : 0) + (longform ? 20 : 0);
+    maxcmd -= 26 + (from ? 17 : 0) + (longform ? 21 : 0);
     if (maxcmd < 3)
 	fprintf(stderr, "warning: screen width %d suboptimal.\n", win.ws_col);
 
@@ -95,14 +95,15 @@ int main(int argc, char **argv) {
 
     if (header) {				/* print uptime and headers */
 	print_uptime();
-	printf("USER     TTY      ");
+	printf("USER      TTY   ");
 	if (from)
 	    printf("FROM             ");
 	if (longform)
 	    printf(" LOGIN@  IDLE   JCPU   PCPU  WHAT\n");
 	else
-	    printf("  IDLE   WHAT\n");
+	    printf("  IDLE  WHAT\n");
     }
+#if 0
     /* open + mmap utmp.  Sure I could us getutent, but I looked at
      * the libc code and it didn't seem to add any real value. */
     if ((fd = open(UTMP_FILE, O_RDONLY)) < 0 ||
@@ -118,6 +119,15 @@ int main(int argc, char **argv) {
                 (user ? !strncmp(u->ut_user, user, strlen(u->ut_user)) : *u->ut_user))
                 showinfo(u, longform, maxcmd, from);
     }
+#else 
+    setutent();
+    while ((u=getutent()) != NULL) {
+      if (u->ut_type == USER_PROCESS && 
+          (user ? !strncmp(u->ut_user, user, strlen(u->ut_user)) : *u->ut_user))
+        showinfo(u, longform, maxcmd, from);
+    }
+    endutent();
+#endif
     return 0;
 }
 
@@ -149,7 +159,7 @@ time_t idletime(utmp_t *u, char *tty) {
 }
 
 void showinfo(utmp_t *u, int formtype, int maxcmd, int from) {
-    int jcpu, i, ut_pid_found;
+    int jcpu, i, ut_pid_found, n,len;
     char uname[USERSZ + 1] = "",
 	tty[5 + sizeof u->ut_line + 1] = "/dev/";
     proc_t *best;
@@ -171,7 +181,7 @@ void showinfo(utmp_t *u, int formtype, int maxcmd, int from) {
 
     strncpy(uname, u->ut_user, USERSZ);		/* force NUL term for printf */
     if (formtype) {
-	printf("%-9.8s%-9.8s", uname, u->ut_line);
+	printf("%-9.8s%-7.7s", uname, u->ut_line);
 	if (from)
 	    print_host(u->ut_host, sizeof u->ut_host + 1);
 	print_logintime(u->ut_time, stdout);
@@ -180,45 +190,49 @@ void showinfo(utmp_t *u, int formtype, int maxcmd, int from) {
 	else {
             putc(' ',stdout);
             if (new_time) {
-                print_time_ival(idletime(u, tty), 7, 0);
+                n=print_time_ival(idletime(u, tty), 6, 0);
             } else {
-                print_time_old_ival(idletime(u, tty), 7);
+                n=print_time_old_ival(idletime(u, tty), 6);
             }
-            
+            if (n<0) maxcmd+=(6+n);
         }
         putc(' ',stdout);
         if (new_time){
-            print_time_ival(jcpu/HZ, 7, (jcpu%HZ) * 100 / HZ);
+            print_time_ival(jcpu/HZ, 6, (jcpu%HZ) * 100 / HZ);
         } else {
-            print_time_old_ival(jcpu/HZ, 7);
+            print_time_old_ival(jcpu/HZ, 6);
         }
 	if (best) {
             int pcpu = best->utime + best->stime;
 	    putc(' ', stdout);
             if (new_time){
-                print_time_ival(pcpu/HZ, 7, (pcpu%HZ) * 100 / HZ );
+                n=print_time_ival(pcpu/HZ, 6, (pcpu%HZ) * 100 / HZ );
             } else {
-                print_time_old_ival(pcpu/HZ, 7);
+                n=print_time_old_ival(pcpu/HZ, 6);
             }
+            if (n<0) maxcmd+=(6+n);
 	} else
 	    printf("   ?   ");
     } else {
-	printf("%-9.8s%-9.8s", u->ut_user, u->ut_line);
+	printf("%-9.8s%-7.7s", u->ut_user, u->ut_line);
 	if (from)
 	    print_host(u->ut_host, sizeof u->ut_host + 1);
         fputs(" ", stdout);
         if (new_time){
-            print_time_ival(idletime(u, tty), 7, 0);
+            n=print_time_ival(idletime(u, tty), 6, 0);
         } else {
-            print_time_old_ival(idletime(u, tty), 7);
+            n=print_time_old_ival(idletime(u, tty), 6);
         }
+        if (n<0) maxcmd+=(6+n);
     }
     fputs(" ", stdout);
     if (best)
 	if (best->cmdline)
 	    print_strlist(best->cmdline, maxcmd, " ");
-	else
-	    printf("-%-*.*s", maxcmd, maxcmd, best->cmd);
+	else {
+            len= maxcmd-1 > strlen (best->cmd) ? strlen (best->cmd) : maxcmd-1;
+	    printf("-%-*.*s", len, len, best->cmd);
+        }
     else
 	printf("-");
     fputc('\n', stdout);
