@@ -28,7 +28,7 @@ extern char *ctime ();
 
 char *program_name;
 
-int
+void
 main (argc, argv)
      int argc;
 char *argv[];
@@ -40,12 +40,11 @@ char *argv[];
   int count = 1;
   long timer;
   int nsecs = 2;
-  int total_chars;
 
   program_name = argv[0];
   if (argc < 2)
     {
-      fprintf (stderr, "Usage: %s COMMAND [-n SLEEP] [ARG ...]\n", argv[0]);
+      fprintf (stderr, "Usage: %s [-n SLEEP] COMMAND [ARGS ...]\n", argv[0]);
       exit (1);
     }
 
@@ -58,22 +57,19 @@ char *argv[];
       count = 3;
       if (nsecs == 0 || argc < 4)
 	{
-	  fprintf (stderr, "Usage: %s COMMAND [-n SLEEP] [ARG ...]\n", argv[0]);
+	  fprintf (stderr, "Usage: %s [-n SLEEP] COMMAND [ARG ...]\n", argv[0]);
 	  exit (1);
 	}
     }
 
   /* Build command string to give to popen.  */
   
-  total_chars = 0;
   bzero (cmd, sizeof (cmd));
-  strncpy (cmd, argv[count], sizeof (cmd));
-  total_chars = strlen(argv[count]);
-  while (++count < argc && total_chars < (sizeof(cmd) - 1)) /* 1 for space */
+  strcpy (cmd, argv[count]);
+  while (++count < argc)
     {
       strcat (cmd, " ");
-      strncat (cmd, argv[count], sizeof(cmd) - total_chars);
-      total_chars += strlen(argv[count]) + 1; /* 1 for space */
+      strcat (cmd, argv[count]);
     }
 
   /* Catch keyboard interrupts so we can put tty back in a sane state.  */
@@ -133,7 +129,7 @@ char *argv[];
   echo ();
   nl ();
   endwin ();
-  return (0);
+  exit (0);
 }
 
 /*------------------------------------.
